@@ -1,5 +1,5 @@
 /*  Copyright 1991 Free Software Foundation, Inc.
- *  Copyright 1997,1999-2002,2007-2009 Alain Knaff.
+ *  Copyright 1997,1999-2002,2007-2009,2022 Alain Knaff.
  *  This file is part of mtools.
  *
  *  Mtools is free software: you can redistribute it and/or modify
@@ -43,7 +43,29 @@ char *strdup(const char *str)
 }
 #endif /* HAVE_STRDUP */
 
+#ifndef HAVE_STRNDUP
+char *strndup( const char *s, size_t n )
+{
+    size_t nAvail;
+    char *p;
+
+    if ( !s )
+        return 0;
+
+    nAvail = min( strlen(s) + 1, n + 1 );
+    p      = malloc( nAvail );
+    if ( !p )
+	    return 0;
+    memcpy( p, s, nAvail );
+    p[nAvail - 1] = '\0';
+
+    return p;
+}
+#endif /* HAVE_STRNDUP */
+
+
 #ifdef HAVE_WCHAR_H
+#include "file_name.h"
 #ifndef HAVE_WCSDUP
 wchar_t *wcsdup(const wchar_t *wcs)
 {
@@ -102,7 +124,7 @@ char * strchr (const char* s, int c)
 {
 	if (!s) return NULL;
 	while (*s && *s != c) s++;
-	if (*s) 
+	if (*s)
 		return (char*) s;
 	else
 		return NULL;
@@ -112,7 +134,7 @@ char * strchr (const char* s, int c)
 
 #ifndef HAVE_STRRCHR
 
-char * strrchr (const char* s1, int c) 
+char * strrchr (const char* s1, int c)
 {
 	char* s = (char*) s1;
 	char* start = (char*) s;
@@ -127,6 +149,53 @@ char * strrchr (const char* s1, int c)
 
 #endif
 
+#ifndef HAVE_STRSTR
+char * strstr (const char* haystack, const char *needle)
+{
+	const char *start;
+	int i;
+	if (!haystack) return NULL;
+	for(start=haystack; *start;start++) {
+		for(i=0; start[i] && needle[i]; i++)
+			if(start[i] != needle[i])
+				break;
+		if(!needle[i])
+			return (char *)start;
+	}
+	return NULL;
+}
+#endif
+
+#ifndef HAVE_MKDIR
+int mkdir (const char* file, int mode)
+{
+	pid_t pid;
+	int stat_loc;
+	
+	switch(pid=fork()) {
+	case 0: /* in the child */
+		execl("/bin/mkdir", "mkdir", file, 0);
+		perror("execl");
+		exit(127);
+		break;
+	case -1:
+		perror("fork");
+		return -1;
+	default:
+		if(wait(&stat_loc) <0) {
+			perror("wait");
+			return -1;
+		} else if((stat_loc & 0xffff) != 0) {
+			fprintf(stderr, "wait returned %x\n",
+				stat_loc & 0xffff);
+			return -1;
+		}
+	}
+	return 0;
+}
+#endif
+
+
 #ifndef HAVE_STRPBRK
 /*
  * Return ptr to first occurrence of any character from `brkset'
@@ -154,7 +223,7 @@ char *strpbrk(const char *string, const char *brkset)
 static int getdigit(char a, int max)
 {
 	int dig;
-	
+
 	if(a < '0')
 		return -1;
 	if(a <= '9') {
@@ -163,6 +232,8 @@ static int getdigit(char a, int max)
 		dig = a - 'a' + 10;
 	else if(a >= 'A')
 		dig = a - 'A' + 10;
+	else
+		return -1;
 	if(dig >= max)
 		return -1;
 	else
@@ -248,7 +319,7 @@ size_t strspn(const char *s, const char *accept)
 #endif /* HAVE_STRSPN */
 
 #ifndef HAVE_STRCSPN
-/* Return the length of the maximum inital segment of S
+/* Return the length of the maximum initial segment of S
    which contains no characters from REJECT.  */
 size_t strcspn (const char *s, const char *reject)
 {
@@ -267,7 +338,7 @@ size_t strcspn (const char *s, const char *reject)
 
 #ifndef HAVE_STRERROR
 
-#ifndef DECL_SYS_ERRLIST
+#if !HAVE_DECL_SYS_ERRLIST
 extern char *sys_errlist[];
 #endif
 
@@ -391,7 +462,7 @@ static exitCallback_t *callback = 0;
 int atexit(void (*function) (void))
 {
 	exitCallback_t *newCallback;
-		
+
 	newCallback = New(exitCallback_t);
 	if(!newCallback) {
 		printOom();
