@@ -20,12 +20,8 @@
  */
 
 
-#define LOWERCASE
-
 #include "sysincludes.h"
-#include "msdos.h"
 #include "mtools.h"
-#include "vfat.h"
 #include "mainloop.h"
 #include "plain_io.h"
 #include "nameclash.h"
@@ -52,7 +48,7 @@ static int dos_showfat(direntry_t *entry, MainParam_t *mp)
 	return GOT_ONE;
 }
 
-static int unix_showfat(MainParam_t *mp)
+static int unix_showfat(MainParam_t *mp UNUSEDP)
 {
 	fprintf(stderr,"File does not reside on a Dos fs\n");
 	return ERROR_ONE;
@@ -69,11 +65,12 @@ static void usage(int ret)
 	exit(ret);
 }
 
-void mshowfat(int argc, char **argv, int mtype)
+void mshowfat(int argc, char **argv, int mtype UNUSEDP) NORETURN;
+void mshowfat(int argc, char **argv, int mtype UNUSEDP)
 {
 	Arg_t arg;
 	int c, ret;
-	
+
 	/* get command line options */
 	if(helpFlag(argc, argv))
 		usage(0);
@@ -84,13 +81,12 @@ void mshowfat(int argc, char **argv, int mtype)
 				arg.offset = str_to_offset(optarg);
 				break;
 			case 'i':
-				set_cmd_line_image(optarg, 0);
+				set_cmd_line_image(optarg);
 				break;
 			case 'h':
 				usage(0);
 			case '?':
 				usage(1);
-				break;
 		}
 	}
 
